/*
   Copyright (c) 2009-2013, Jack Poulson
   All rights reserved.

   This file is part of Elemental and is under the BSD 2-Clause License, 
   which can be found in the LICENSE file in the root directory, or at 
   http://opensource.org/licenses/BSD-2-Clause
*/
#ifndef EL_CONFIG_H
#define EL_CONFIG_H

/* Build type and version information */
#define EL_GIT_SHA1 "GITDIR-NOTFOUND"
#define EL_VERSION_MAJOR "0"
#define EL_VERSION_MINOR "85"
#define EL_CMAKE_BUILD_TYPE "PureRelease"
#define EL_RELEASE

#define EL_MPI_LINK_FLAGS      " -Wl,-z,noexecstack  -Wl,-rpath  -Wl,/usr/lib64/mpich/lib"

/* C compiler info */
#define EL_CMAKE_C_COMPILER    "/usr/bin/cc"
#define EL_MPI_C_COMPILER      "/usr/lib64/mpich/bin/mpicc"
#define EL_MPI_C_INCLUDE_PATH  "/usr/include/mpich-x86_64"
#define EL_MPI_C_COMPILE_FLAGS " -fexceptions  -fstack-protector-strong  -fPIC"
#define EL_MPI_C_LIBRARIES     "/usr/lib64/mpich/lib/libmpich.so;/usr/lib64/mpich/lib/libopa.so;/usr/lib64/mpich/lib/libmpl.so;/usr/lib64/librt.so;/usr/lib64/libpthread.so"

/* C++ compiler info */
#define EL_CMAKE_CXX_COMPILER    "/usr/bin/c++"
#define EL_CXX_FLAGS             "-O3 -std=c++11  -fexceptions  -fstack-protector-strong  -fPIC"
#define EL_MPI_CXX_COMPILER      "/usr/lib64/mpich/bin/mpicxx"
#define EL_MPI_CXX_INCLUDE_PATH  "/usr/include/mpich-x86_64"
#define EL_MPI_CXX_COMPILE_FLAGS " -fexceptions  -fstack-protector-strong  -fPIC"
#define EL_MPI_CXX_LIBRARIES     "/usr/lib64/mpich/lib/libmpichcxx.so;/usr/lib64/mpich/lib/libmpich.so;/usr/lib64/mpich/lib/libopa.so;/usr/lib64/mpich/lib/libmpl.so;/usr/lib64/librt.so;/usr/lib64/libpthread.so"
/* #undef EL_MPI_COMM_IS_VOIDP */
/* #undef EL_MPI_GROUP_IS_VOIDP */

/* Math libraries */
#define EL_MATH_LIBS "-lopenblas"
#define EL_BLAS_POST
#define EL_LAPACK_POST
/* #undef EL_HAVE_SCALAPACK */
/* #undef EL_SCALAPACK_POST */
/* #undef EL_HAVE_FLA_BSVD */
#define EL_FORT_LOGICAL int
#define EL_FORT_TRUE    1
#define EL_FORT_FALSE   0

/* Basic configuration options */
#define EL_RESTRICT __restrict__
/* #undef EL_HAVE_OPENMP */
#define EL_HAVE_OMP_COLLAPSE
/* #undef EL_HAVE_QT5 */
#define EL_HAVE_F90_INTERFACE
#define EL_AVOID_COMPLEX_MPI
#define EL_HAVE_CXX11RANDOM
#define EL_HAVE_STEADYCLOCK
#define EL_HAVE_NOEXCEPT
#define EL_HAVE_MPI_REDUCE_SCATTER_BLOCK
#define EL_HAVE_MPI_IN_PLACE
#define EL_HAVE_MPI_LONG_LONG
#define EL_HAVE_MPI_COMM_SET_ERRHANDLER
#define EL_HAVE_MPI_INIT_THREAD
#define EL_HAVE_MPI_QUERY_THREAD
#define EL_HAVE_MPI3_NONBLOCKING_COLLECTIVES
/* #undef EL_HAVE_MPIX_NONBLOCKING_COLLECTIVES */
/* #undef EL_REDUCE_SCATTER_BLOCK_VIA_ALLREDUCE */
#define EL_USE_BYTE_ALLGATHERS
/* #undef EL_USE_64BIT_INTS */

/* Sparse-direct configuration */
#define EL_USE_CUSTOM_ALLTOALLV
/* #undef EL_BARRIER_IN_ALLTOALLV */
#define EL_HAVE_METIS
/* #undef EL_HAVE_PARMETIS */

/* Advanced configuration options */
/* #undef EL_ZERO_INIT */
/* #undef EL_CACHE_WARNINGS */
/* #undef EL_UNALIGNED_WARNINGS */
/* #undef EL_VECTOR_WARNINGS */
/* #undef EL_POOL_MEMORY */
/* #undef EL_AVOID_OMP_FMA */

/* #undef EL_DECLSPEC */
#ifdef EL_DECLSPEC
# define EL_EXPORT __declspec(dllexport)
#else
# define EL_EXPORT
#endif

/* NOTE: Only EL_DISABLE_FLOAT has been tested so far */
/* #undef EL_DISABLE_FLOAT */
/* #undef EL_DISABLE_DOUBLE */
/* #undef EL_DISABLE_COMPLEX */

/* #undef EL_HAVE_VALGRIND */

#endif /* EL_CONFIG_H */
